@@ -13,7 +13,7 @@ if [ -n "$GNOME2_DIR" ]; then
     export LD_LIBRARY_PATH
 fi
 
-(test -f $srcdir/configure.in) || {
+(test -f $srcdir/configure.ac) || {
     echo -n "**Error**: Directory "\`$srcdir\'" does not look like the"
     echo " top-level package directory"
     exit 1
@@ -27,7 +27,7 @@ fi
   DIE=1
 }
 
-(grep "^IT_PROG_INTLTOOL" $srcdir/configure.in >/dev/null) && {
+(grep "^IT_PROG_INTLTOOL" $srcdir/configure.ac >/dev/null) && {
   (intltoolize --version) < /dev/null > /dev/null 2>&1 || {
     echo 
     echo "**Error**: You must have \`intltool' installed."
@@ -37,7 +37,7 @@ fi
   }
 }
 
-(grep "^AM_PROG_XML_I18N_TOOLS" $srcdir/configure.in >/dev/null) && {
+(grep "^AM_PROG_XML_I18N_TOOLS" $srcdir/configure.ac >/dev/null) && {
   (xml-i18n-toolize --version) < /dev/null > /dev/null 2>&1 || {
     echo
     echo "**Error**: You must have \`xml-i18n-toolize' installed."
@@ -47,7 +47,7 @@ fi
   }
 }
 
-(grep "^AM_PROG_LIBTOOL" $srcdir/configure.in >/dev/null) && {
+(grep "^AM_PROG_LIBTOOL" $srcdir/configure.ac >/dev/null) && {
   (${LIBTOOL:-libtool} --version) < /dev/null > /dev/null 2>&1 || {
     echo
     echo "**Error**: You must have \`libtool' installed."
@@ -56,8 +56,8 @@ fi
   }
 }
 
-(grep "^AM_GLIB_GNU_GETTEXT" $srcdir/configure.in >/dev/null) && {
-  (grep "sed.*POTFILES" $srcdir/configure.in) > /dev/null || \
+(grep "^AM_GLIB_GNU_GETTEXT" $srcdir/configure.ac >/dev/null) && {
+  (grep "sed.*POTFILES" $srcdir/configure.ac) > /dev/null || \
   (glib-gettextize --version) < /dev/null > /dev/null 2>&1 || {
     if [ -f /etc/debian_version ]
     then
@@ -106,7 +106,7 @@ xlc )
   am_opt=--include-deps;;
 esac
 
-for coin in `find $srcdir -name configure.in -print`
+for coin in `find $srcdir -name configure.ac -print`
 do 
   dr=`dirname $coin`
   if test -f $dr/NO-AUTO-GEN; then
@@ -117,7 +117,7 @@ do
 
       aclocalinclude="$ACLOCAL_FLAGS"
 
-      if grep "^AM_GLIB_GNU_GETTEXT" configure.in >/dev/null; then
+      if grep "^AM_GLIB_GNU_GETTEXT" configure.ac >/dev/null; then
         echo "Creating $dr/aclocal.m4 ..."
         test -r $dr/aclocal.m4 || touch $dr/aclocal.m4
         echo "Running glib-gettextize...  Ignore non-fatal messages."
@@ -125,15 +125,15 @@ do
         echo "Making $dr/aclocal.m4 writable ..."
         test -r $dr/aclocal.m4 && chmod u+w $dr/aclocal.m4
       fi
-      if grep "^IT_PROG_INTLTOOL" configure.in >/dev/null; then
+      if grep "^IT_PROG_INTLTOOL" configure.ac >/dev/null; then
         echo "Running intltoolize..."
         intltoolize --copy --force --automake
       fi
-      if grep "^AM_PROG_XML_I18N_TOOLS" configure.in >/dev/null; then
+      if grep "^AM_PROG_XML_I18N_TOOLS" configure.ac >/dev/null; then
         echo "Running xml-i18n-toolize..."
         xml-i18n-toolize --copy --force --automake
       fi
-      if grep "^AM_PROG_LIBTOOL" configure.in >/dev/null; then
+      if grep "^A[CM]_PROG_LIBTOOL" configure.ac >/dev/null; then
         if test -z "$NO_LIBTOOLIZE" ; then 
           echo "Running libtoolize..."
           ${LIBTOOLIZE:-libtoolize} --force --copy
@@ -141,14 +141,17 @@ do
       fi
       echo "Running aclocal $aclocalinclude ..."
       ${ACLOCAL:-aclocal} $aclocalinclude
-      if grep "^AM_CONFIG_HEADER" configure.in >/dev/null; then
+      if grep "^AM_CONFIG_HEADER" configure.ac >/dev/null; then
         echo "Running autoheader..."
         ${AUTOHEADER:-autoheader}
       fi
       echo "Running automake --gnu $am_opt ..."
       ${AUTOMAKE:-automake} --add-missing --gnu $am_opt
       echo "Running autoconf ..."
-      ${AUTOCONF:-autoconf}
+			# Needs to have force to recreate the configure script even if
+			# it exists so the correct git version is applied as the version
+			# number.
+      (${AUTOCONF:-autoconf} --force)
     )
   fi
 done
