@@ -32,6 +32,7 @@
 #include "AtomicParsley.h"
 #include "AP_AtomExtracts.h"
 #include "AtomicParsleyBridge.h"
+#include "AtomicParsley_genres.h"
 
 extern "C" {
 #include <glib/gstdio.h>
@@ -72,6 +73,12 @@ extern "C" {
 #define PODCAST_URL "purl"
 #define ARTWORK "covr"
 #define GAPLESS_FLAG "pgap"
+#define SORT_TITLE "sonm"
+#define SORT_ARTIST "soar"
+#define SORT_ALBUM_ARTIST "soaa"
+#define SORT_COMPOSER "soco"
+#define SORT_ALBUM "soal"
+#define SORT_TV_SHOW "sosn"
 
 static guint32 mediaTypeTagToMediaType(guint8 media_type) {
     switch (media_type) {
@@ -140,7 +147,10 @@ void AP_read_metadata(const char *filePath, Track *track) {
     FILE *mp4File;
     Trackage *trackage;
     uint8_t track_cur;
+    uint8_t txttrack_cur;
     gboolean audio_or_video_found = FALSE;
+    gboolean has_quicktime_chaps = FALSE;
+    uint32_t timescale = 0;
 
     APar_ScanAtoms(filePath, true);
     mp4File = openSomeFile(filePath, true);
@@ -150,20 +160,135 @@ void AP_read_metadata(const char *filePath, Track *track) {
     for (track_cur = 0; track_cur < trackage->total_tracks; ++track_cur) {
         TrackInfo *info = trackage->infos[track_cur];
 
-        // FIXME no chapter information implemented yet
+        if ((info->type_of_track & AUDIO_TRACK) || (info->type_of_track & VIDEO_TRACK)
+                        || (info->type_of_track & DRM_PROTECTED_TRACK)) {
 
-        if (info->track_type && audio_or_video_found == FALSE
-                && ((info->track_type & AUDIO_TRACK) || (info->track_type & VIDEO_TRACK)
-                        || (info->track_type & DRM_PROTECTED_TRACK))) {
+            /*
+             * the info->duration is in the track's timescale units so must be divided by that
+             * value to get seconds, while track->tracklen in gtkpod is in ms
+             */
+            float duration = ((float) info->duration / (float) info->parent->movie_info->timescale) * 1000;
+            track->tracklen = (gint32) duration;
 
-            track->tracklen = info->duration;
             track->bitrate = APar_calculate_bitrate(info);
             track->samplerate = info->media_sample_rate;
+            audio_or_video_found = TRUE;
+            break;
         }
-
-        audio_or_video_found = TRUE;
-
     }
+    for (txttrack_cur = 0; audio_or_video_found && txttrack_cur < trackage->total_tracks; ++txttrack_cur) {
+        TrackInfo *txtinfo = trackage->infos[txttrack_cur];
+        char buf[128];
+        // search for chapter track
+        if (!(txtinfo->type_of_track & TEXT_TRACK))
+            continue;
+        // see if the AV track's chap refers to this text track
+        // chap: 0: atom size  4: 'chap'  8,12,...,8+(N-1)*4: (0: referenced track ID)
+        snprintf(buf, sizeof(buf), "moov.trak[%u].tref.chap", track_cur + 1);
+        AtomicInfo* chapAtom = APar_FindAtom(buf, false, SIMPLE_ATOM, 0);
+        if (!chapAtom)
+            continue;
+        int entry_count = (chapAtom->AtomicLength - 8) / 4;
+        for (int i = 0; i < entry_count; ++i) {
+            if (APar_read32(buf, mp4File, chapAtom->AtomicStart + 8 + i * 4) == txtinfo->track_id) {
+                has_quicktime_chaps = TRUE;
+                timescale = txtinfo->media_sample_rate;
+                break;
+            }
+        }
+        if (has_quicktime_chaps)
+            break;
+    }
+    if (has_quicktime_chaps) {
+        // found a chapter... now get the chapter data from the text track
+        char buf[128];
+
+        // stts: 0: atom size  4: 'stts'  8: version  12: entry count  16,24,...,16+(N-1)*8: (0: frame count 4: duration)
+        snprintf(buf, sizeof(buf), "moov.trak[%u].mdia.minf.stbl.stts", txttrack_cur + 1);
+        AtomicInfo* sampleAtom = APar_FindAtom(buf, false, VERSIONED_ATOM, 0);
+
+        // stsz: 0: atom size  4: 'stsz'  8: version  12: size of all (or 0)  16: entry count  20,24,...,20+(N-1)*4: (0: sample size)
+        snprintf(buf, sizeof(buf), "moov.trak[%u].mdia.minf.stbl.stsz", txttrack_cur + 1);
+        AtomicInfo* sampleSizeAtom = APar_FindAtom(buf, false, VERSIONED_ATOM, 0);
+
+        // stco: 0: atom size  4: 'stco'  8: version  12: entry count  16,20,...,16+(N-1)*4: (0: sample byte offset)
+        snprintf(buf, sizeof(buf), "moov.trak[%u].mdia.minf.stbl.stco", txttrack_cur + 1);
+        AtomicInfo* sampleOffsetAtom = APar_FindAtom(buf, false, VERSIONED_ATOM, 0);
+
+        // We must have a valid sampleAtom to know chapter times. If sampleSizeAtom or sampleOffsetAtom is invalid,
+        // we can do without them (and instead create a default chapter name).
+        if (sampleAtom && sampleAtom->AtomicLength >= 16) {
+            Itdb_Chapterdata *chapterdata = itdb_chapterdata_new();
+            uint32_t stts_entry_count = APar_read32(buf, mp4File, sampleAtom->AtomicStart + 12);
+            uint32_t stsz_entry_count = !sampleSizeAtom || sampleSizeAtom->AtomicLength < 20 ? 0 :
+                APar_read32(buf, mp4File, sampleSizeAtom->AtomicStart + 16);
+            uint32_t stco_entry_count = !sampleOffsetAtom || sampleOffsetAtom->AtomicLength < 16 ? 0 :
+                APar_read32(buf, mp4File, sampleOffsetAtom->AtomicStart + 12);
+            uint32_t stsz_all_size = !sampleSizeAtom || sampleSizeAtom->AtomicLength < 16 ? 0 :
+                APar_read32(buf, mp4File, sampleSizeAtom->AtomicStart + 12);
+
+            uint32_t start_time = 0;
+
+            u_int32_t max_frame_size = stsz_all_size; // if stsz_all_size specified, use only that size
+            for (int i = 0; !stsz_all_size && i < stsz_entry_count; ++i) {
+                uint32_t chap_name_len = APar_read32(buf, mp4File, sampleSizeAtom->AtomicStart + 20 + i * 4);
+                if (chap_name_len > max_frame_size)
+                    max_frame_size = chap_name_len;
+            }
+            max_frame_size += 1; // for trailing '\0' (unneeded?), and to make sure that malloc() gets passed at least 1
+            char * namebuf = (char *)malloc(max_frame_size * sizeof(char));
+            for (int i = 0; i < stts_entry_count; ++i) {
+                gchar *title = NULL;
+                uint32_t chap_name_len = stsz_all_size;
+                uint32_t chap_offset = 0;
+                if (stsz_all_size == 0 && i < stsz_entry_count)
+                    chap_name_len = APar_read32(buf, mp4File, sampleSizeAtom->AtomicStart + 20 + i * 4);
+                if (i < stco_entry_count)
+                    chap_offset = APar_read32(buf, mp4File, sampleOffsetAtom->AtomicStart + 16 + i * 4);
+                if (chap_offset != 0)
+                    APar_readX(namebuf, mp4File, chap_offset, chap_name_len);
+                else // If the location of the chapter name is unknown, trigger default chapter naming
+                    chap_name_len = 0;
+                if (chap_name_len > 2) {
+                    int titlelength = (namebuf[0] << 8) + namebuf[1];
+                    // if the stsz atom and the title value disagree, use the smaller one for safety
+                    titlelength = (titlelength > chap_name_len) ? chap_name_len : titlelength;
+                    // If a title begins with 0xFFFE, it's a UTF-16 title
+                    if (titlelength >= 2 && namebuf[2] == 0xff && namebuf[3] == 0xfe)
+                        title = g_utf16_to_utf8((const gunichar2 *) &namebuf[4], titlelength - 2, NULL, NULL, NULL);
+                    else
+                        title = g_strndup(&namebuf[2], titlelength);
+                }
+                else
+                {
+                    // chapter title couldn't be found; create our own titles (and some ipods don't display them anyway).
+                    // Translators: this string is used to create a chapter title when no chapter title could be found
+                    title = g_strdup_printf(_("Chapter %3d"), i);
+                }
+
+                if (!timescale) // assume 1000, also, don't divide by 0
+                    timescale = 1000;
+                double duration_ms = (double)start_time * 1000.0 / (double)timescale;
+
+                itdb_chapterdata_add_chapter(chapterdata, duration_ms, title);
+                g_free(title);
+
+                if (i < (stts_entry_count - 1)) // skip this stage after the last chapter has been added
+                {
+                    uint32_t frame_count = APar_read32(buf, mp4File, sampleAtom->AtomicStart + 16 + i * 8);
+                    uint32_t duration = APar_read32(buf, mp4File, sampleAtom->AtomicStart + 20 + i * 8);
+                    start_time += frame_count * duration;
+                }
+            }
+            if (namebuf)
+                free(namebuf);
+            if (track->chapterdata) // if there was already chapter data, don't leak it
+                itdb_chapterdata_free(track->chapterdata);
+            track->chapterdata = itdb_chapterdata_duplicate(chapterdata);
+            itdb_chapterdata_free(chapterdata);
+        }
+    }
+    // TODO: add support for Nero-style mp4 chapters
 
     if (prefs_get_int("readtags")) {
         char* value = NULL;
@@ -171,49 +296,49 @@ void AP_read_metadata(const char *filePath, Track *track) {
         // MP4 Title
         value = find_atom_value(TITLE);
         if (value) {
-            track->title = charset_to_utf8(value);
+            track->title = g_strdup(value);
             free(value);
         }
 
         // MP4 Artist
         value = find_atom_value(ARTIST);
         if (value) {
-            track->artist = charset_to_utf8(value);
+            track->artist = g_strdup(value);
             free(value);
         }
 
         // MP4 Album Artist
         value = find_atom_value(ALBUM_ARTIST);
         if (value) {
-            track->albumartist = charset_to_utf8(value);
+            track->albumartist = g_strdup(value);
             free(value);
         }
 
         // MP4 Composer
         value = find_atom_value(COMPOSER);
         if (value) {
-            track->composer = charset_to_utf8(value);
+            track->composer = g_strdup(value);
             free(value);
         }
 
         // MP4 Comment
         value = find_atom_value(COMMENT);
         if (value) {
-            track->comment = charset_to_utf8(value);
+            track->comment = g_strdup(value);
             free(value);
         }
 
         // MP4 Description
         value = find_atom_value(DESCRIPTION);
         if (value) {
-            track->description = charset_to_utf8(value);
+            track->description = g_strdup(value);
             free(value);
         }
 
         // MP4 Keywords
         value = find_atom_value(KEYWORD);
         if (value) {
-            track->keywords = charset_to_utf8(value);
+            track->keywords = g_strdup(value);
             free(value);
         }
 
@@ -227,7 +352,7 @@ void AP_read_metadata(const char *filePath, Track *track) {
         // MP4 Album
         value = find_atom_value(ALBUM);
         if (value) {
-            track->album = charset_to_utf8(value);
+            track->album = g_strdup(value);
             free(value);
         }
 
@@ -266,7 +391,7 @@ void AP_read_metadata(const char *filePath, Track *track) {
         // MP4 Grouping
         value = find_atom_value(GROUPING);
         if (value) {
-            track->grouping = charset_to_utf8(value);
+            track->grouping = g_strdup(value);
             free(value);
         }
 
@@ -281,7 +406,7 @@ void AP_read_metadata(const char *filePath, Track *track) {
             // custom genre
             value = find_atom_value(CUSTOM_GENRE);
             if (value) {
-                track->genre = charset_to_utf8(value);
+                track->genre = g_strdup(value);
                 free(value);
             }
         }
@@ -303,14 +428,14 @@ void AP_read_metadata(const char *filePath, Track *track) {
         // MP4 TV Show
         value = find_atom_value(TV_SHOW);
         if (value) {
-            track->tvshow = charset_to_utf8(value);
+            track->tvshow = g_strdup(value);
             free(value);
         }
 
         // MP4 TV Episode
         value = find_atom_value(TV_EPISODE);
         if (value) {
-            track->tvepisode = charset_to_utf8(value);
+            track->tvepisode = g_strdup(value);
             free(value);
         }
 
@@ -324,7 +449,7 @@ void AP_read_metadata(const char *filePath, Track *track) {
         // MP4 TV Network
         value = find_atom_value(TV_NETWORK_NAME);
         if (value) {
-            track->tvnetwork = charset_to_utf8(value);
+            track->tvnetwork = g_strdup(value);
             free(value);
         }
 
@@ -338,21 +463,25 @@ void AP_read_metadata(const char *filePath, Track *track) {
         // MP4 Media Type
         value = find_atom_value(MEDIA_TYPE);
         if (value) {
-            track->mediatype = mediaTypeTagToMediaType(atoi(value));
+            stiks * stik = MatchStikString(value);
+            if (stik)
+            {
+                track->mediatype = mediaTypeTagToMediaType(stik->stik_number);
+            }
             // Should not free standard media types
         }
 
         // MP4 Compilation flag
         value = find_atom_value(COMPILATION);
         if (value) {
-            track->compilation = atoi(value);
+            track->compilation = !g_strcmp0("true", value);
             free(value);
         }
 
         // MP4 Category
         value = find_atom_value(CATEGORY);
         if (value) {
-            track->category = charset_to_utf8(value);
+            track->category = g_strdup(value);
             free(value);
         }
 
@@ -369,6 +498,48 @@ void AP_read_metadata(const char *filePath, Track *track) {
             free(value);
         }
 
+        // MP4 Sort Title
+        value = find_atom_value(SORT_TITLE);
+        if (value) {
+            track->sort_title = g_strdup(value);
+            free(value);
+        }
+
+        // MP4 Sort Artist
+        value = find_atom_value(SORT_ARTIST);
+        if (value) {
+            track->sort_artist = g_strdup(value);
+            free(value);
+        }
+
+        // MP4 Sort Album Artist
+        value = find_atom_value(SORT_ALBUM_ARTIST);
+        if (value) {
+            track->sort_albumartist = g_strdup(value);
+            free(value);
+        }
+
+        // MP4 Sort Composer
+        value = find_atom_value(SORT_COMPOSER);
+        if (value) {
+            track->sort_composer = g_strdup(value);
+            free(value);
+        }
+
+        // MP4 Sort Album
+        value = find_atom_value(SORT_ALBUM);
+        if (value) {
+            track->sort_album = g_strdup(value);
+            free(value);
+        }
+
+        // MP4 Sort TV Show
+        value = find_atom_value(SORT_TV_SHOW);
+        if (value) {
+            track->sort_tvshow = g_strdup(value);
+            free(value);
+        }
+
         if (prefs_get_int("coverart_apic")) {
             gchar *tmp_file_prefix = g_build_filename(g_get_tmp_dir(), "ttt", NULL);
             gchar *tmp_file;
@@ -644,6 +815,24 @@ void AP_write_metadata(Track *track, const char *filePath, GError **error) {
     }
     g_free(atom);
 
+    // Sort Title
+    set_limited_text_atom_value(SORT_TITLE, track->sort_title);
+
+    // Sort Artist
+    set_limited_text_atom_value(SORT_ARTIST, track->sort_artist);
+
+    // Sort Album Artist
+    set_limited_text_atom_value(SORT_ALBUM_ARTIST, track->sort_albumartist);
+
+    // Sort Composer
+    set_limited_text_atom_value(SORT_COMPOSER, track->sort_composer);
+
+    // Sort Album
+    set_limited_text_atom_value(SORT_ALBUM, track->sort_album);
+
+    // Sort TV Show
+    set_limited_text_atom_value(SORT_TV_SHOW, track->sort_tvshow);
+
     if (prefs_get_int("coverart_apic")) {
         GdkPixbuf *pixbuf = (GdkPixbuf*) itdb_artwork_get_pixbuf(track->itdb->device, track->artwork, -1, -1);
         if (!pixbuf) {
@@ -661,7 +850,7 @@ void AP_write_metadata(Track *track, const char *filePath, GError **error) {
                 g_remove(tmp_file);
             }
             else {
-                gtkpod_log_error(error, g_strdup_printf(_("ERROR failed to change track file's artwork.")));
+                gtkpod_log_error(error, g_strdup_printf(_("ERROR failed to change track file's artwork.") ));
                 g_error_free(pixbuf_err);
                 return;
             }
