@@ -28,6 +28,7 @@
 
 #include "clarity_utils.h"
 #include "clarity_cover.h"
+#include "clarity_flip_effect.h"
 
 #define V_PADDING 4
 
@@ -38,7 +39,7 @@
  * An artwork cover widget with support for a text label and background color.
  */
 
-G_DEFINE_TYPE( ClarityCover, clarity_cover, CLUTTER_TYPE_GROUP);
+G_DEFINE_TYPE( ClarityCover, clarity_cover, CLUTTER_TYPE_ACTOR);
 
 /* macro for accessing the object's private structure */
 #define CLARITY_COVER_GET_PRIVATE(obj) \
@@ -56,11 +57,12 @@ G_DEFINE_TYPE( ClarityCover, clarity_cover, CLUTTER_TYPE_GROUP);
  * (toggled on or off) or a background image
  */
 struct _ClarityCoverPrivate {
-    ClutterActor *container;
 
     ClutterActor *texture;
+    ClutterContent *artwork;
+    int height;
+    int width;
     ClutterActor *reflection;
-    ClutterBehaviour  *rotateBehaviour;
 
     gchar *title;
     gchar *artist;
@@ -108,13 +110,15 @@ void clarity_cover_destroy(ClutterActor *self) {
             priv->texture = NULL;
         }
 
+        if (CLUTTER_IS_IMAGE(priv->artwork)) {
+            g_object_unref(priv->artwork);
+            priv->artwork = NULL;
+        }
+
         if (CLUTTER_IS_ACTOR(priv->reflection)) {
             clutter_actor_destroy(priv->reflection);
             priv->reflection = NULL;
         }
-
-        if (G_IS_OBJECT(priv->rotateBehaviour))
-            priv->rotateBehaviour = NULL;
     }
 
     /* chain up to destroy() on the parent ClutterActorClass;
@@ -156,74 +160,137 @@ static void clarity_cover_init(ClarityCover *self) {
 
     priv->title = NULL;
     priv->artist = NULL;
-    priv->container = NULL;
 
     priv->texture = NULL;
+    priv->artwork = NULL;
+    priv->width = 0;
+    priv->height = 0;
     priv->reflection = NULL;
 }
 
-static void _clone_paint_cb (ClutterActor *actor) {
-    ClutterActor *source;
-    ClutterActorBox box;
-    CoglHandle material;
-    gfloat width, height;
-    guint8 opacity;
-    CoglColor color_1, color_2;
-    CoglTextureVertex vertices[4];
+//static void _clone_paint_cb (ClutterActor *actor) {
+//    ClutterActor *source;
+//    ClutterActorBox box;
+//    CoglHandle material;
+//    gfloat width, height;
+//    guint8 opacity;
+//    CoglColor color_1, color_2;
+//    CoglTextureVertex vertices[4];
+//
+//    /* if we don't have a source actor, don't paint */
+//    source = clutter_clone_get_source (CLUTTER_CLONE (actor));
+//    if (source == NULL)
+//        goto out;
+//
+//    /* if the source does not have any content, don't paint */
+//    ClutterContent *content = clutter_actor_get_content(actor);
+//    clutter_image_
+//
+//    material = clutter_texture_get_cogl_material (CLUTTER_TEXTURE (source));
+//    if (material == NULL)
+//        goto out;
+//
+//    /* get the size of the reflection */
+//    clutter_actor_get_allocation_box (actor, &box);
+//    clutter_actor_box_get_size (&box, &width, &height);
+//
+//    /* get the composite opacity of the actor */
+//    opacity = clutter_actor_get_paint_opacity (actor);
+//
+//    /* figure out the two colors for the reflection: the first is
+//     * full color and the second is the same, but at 0 opacity
+//     */
+//    cogl_color_init_from_4f (&color_1, 1.0, 1.0, 1.0, opacity / 255.0);
+//    cogl_color_premultiply (&color_1);
+//    cogl_color_init_from_4f (&color_2, 1.0, 1.0, 1.0, 0.0);
+//    cogl_color_premultiply (&color_2);
+//
+//    /* now describe the four vertices of the quad; since it has
+//     * to be a reflection, we need to invert it as well
+//    */
+//    vertices[0].x = 0; vertices[0].y = 0; vertices[0].z = 0;
+//    vertices[0].tx = 0.0; vertices[0].ty = 1.0;
+//    vertices[0].color = color_1;
+//
+//    vertices[1].x = width; vertices[1].y = 0; vertices[1].z = 0;
+//    vertices[1].tx = 1.0; vertices[1].ty = 1.0;
+//    vertices[1].color = color_1;
+//
+//    vertices[2].x = width; vertices[2].y = height; vertices[2].z = 0;
+//    vertices[2].tx = 1.0; vertices[2].ty = 0.0;
+//    vertices[2].color = color_2;
+//
+//    vertices[3].x = 0; vertices[3].y = height; vertices[3].z = 0;
+//    vertices[3].tx = 0.0; vertices[3].ty = 0.0;
+//    vertices[3].color = color_2;
+//
+//    /* paint the same texture but with a different geometry */
+//    cogl_set_source (material);
+//    cogl_polygon (vertices, 4, TRUE);
+//
+//    out:
+//      /* prevent the default clone handler from running */
+//      g_signal_stop_emission_by_name (actor, "paint");
+//}
 
-    /* if we don't have a source actor, don't paint */
-    source = clutter_clone_get_source (CLUTTER_CLONE (actor));
-    if (source == NULL)
-        goto out;
+/**
+ * Create the reflection for the given cover
+ */
+static void _create_reflection(ClarityCover *self, GdkPixbuf *albumart) {
+    GError *error = NULL;
+    gint y_offset;
+    ClutterContent *reflectimg;
 
-    /* if the source texture does not have any content, don't paint */
-    material = clutter_texture_get_cogl_material (CLUTTER_TEXTURE (source));
-    if (material == NULL)
-        goto out;
+    g_return_if_fail(CLARITY_IS_COVER(self));
 
-    /* get the size of the reflection */
-    clutter_actor_get_allocation_box (actor, &box);
-    clutter_actor_box_get_size (&box, &width, &height);
+    ClarityCoverPrivate *priv = CLARITY_COVER_GET_PRIVATE (self);
+    g_return_if_fail(priv);
 
-    /* get the composite opacity of the actor */
-    opacity = clutter_actor_get_paint_opacity (actor);
+    if (! priv->reflection) {
+        priv->reflection = clutter_actor_new();
+        y_offset = clutter_actor_get_height (priv->texture) + V_PADDING;
+        clutter_actor_add_constraint (priv->reflection, clutter_bind_constraint_new (priv->texture, CLUTTER_BIND_X, 0.0));
+        clutter_actor_add_constraint (priv->reflection, clutter_bind_constraint_new (priv->texture, CLUTTER_BIND_Y, y_offset));
+        clutter_actor_add_constraint (priv->reflection, clutter_bind_constraint_new (priv->texture, CLUTTER_BIND_WIDTH, 0.0));
+        clutter_actor_add_constraint (priv->reflection, clutter_bind_constraint_new (priv->texture, CLUTTER_BIND_HEIGHT, 0.0));
 
-    /* figure out the two colors for the reflection: the first is
-     * full color and the second is the same, but at 0 opacity
-     */
-    cogl_color_init_from_4f (&color_1, 1.0, 1.0, 1.0, opacity / 255.0);
-    cogl_color_premultiply (&color_1);
-    cogl_color_init_from_4f (&color_2, 1.0, 1.0, 1.0, 0.0);
-    cogl_color_premultiply (&color_2);
+        ClutterEffect * flip_effect = clarity_flip_effect_new(priv->texture, priv->reflection);
+        clutter_actor_add_effect(priv->reflection, flip_effect);
+        clutter_actor_add_child(CLUTTER_ACTOR(self), priv->reflection);
+    }
 
-    /* now describe the four vertices of the quad; since it has
-     * to be a reflection, we need to invert it as well
-     */
-    vertices[0].x = 0; vertices[0].y = 0; vertices[0].z = 0;
-    vertices[0].tx = 0.0; vertices[0].ty = 1.0;
-    vertices[0].color = color_1;
+    /* flip image vertically to create reflection */
+    GdkPixbuf *reflect = gdk_pixbuf_flip(albumart, FALSE);
+    GdkPixbuf *scaled = gdk_pixbuf_scale_simple(reflect, priv->width, priv->height, GDK_INTERP_BILINEAR);
 
-    vertices[1].x = width; vertices[1].y = 0; vertices[1].z = 0;
-    vertices[1].tx = 1.0; vertices[1].ty = 1.0;
-    vertices[1].color = color_1;
+    // Set cover reflection
+    reflectimg = clutter_image_new();
+    clutter_image_set_data( CLUTTER_IMAGE (reflectimg),
+                                                        gdk_pixbuf_get_pixels (scaled),
+                                                        gdk_pixbuf_get_has_alpha (scaled)
+                                                                        ? COGL_PIXEL_FORMAT_RGBA_8888
+                                                                        : COGL_PIXEL_FORMAT_RGB_888,
+                                                        priv->width,
+                                                        priv->height,
+                                                        gdk_pixbuf_get_rowstride (scaled),
+                                                        &error);
 
-    vertices[2].x = width; vertices[2].y = height; vertices[2].z = 0;
-    vertices[2].tx = 1.0; vertices[2].ty = 0.0;
-    vertices[2].color = color_2;
+    g_object_unref(reflect);
+    g_object_unref(scaled);
 
-    vertices[3].x = 0; vertices[3].y = height; vertices[3].z = 0;
-    vertices[3].tx = 0.0; vertices[3].ty = 0.0;
-    vertices[3].color = color_2;
+    if (error) {
+        g_warning("%s", error->message);
+        g_error_free(error);
+        return;
+    }
 
-    /* paint the same texture but with a different geometry */
-    cogl_set_source (material);
-    cogl_polygon (vertices, 4, TRUE);
-
-    out:
-      /* prevent the default clone handler from running */
-      g_signal_stop_emission_by_name (actor, "paint");
+    // Set the width and height
+    clutter_actor_set_width(priv->reflection, priv->width);
+    clutter_actor_set_height(priv->reflection, priv->height);
+    clutter_actor_set_content (priv->reflection, reflectimg);
 }
 
+
 void clarity_cover_set_album_item (ClarityCover *self, AlbumItem *item) {
     g_return_if_fail(CLARITY_IS_COVER(self));
 
@@ -231,47 +298,48 @@ void clarity_cover_set_album_item (ClarityCover *self, AlbumItem *item) {
     g_return_if_fail(priv);
 
     GError *error = NULL;
-    gint y_offset;
 
     if (!priv->texture) {
-        priv->texture = gtk_clutter_texture_new();
-        clutter_container_add_actor(CLUTTER_CONTAINER(self), priv->texture);
+        priv->texture = clutter_actor_new();
+        clutter_actor_add_child(CLUTTER_ACTOR(self), priv->texture);
+    }
+
+    if (!priv->artwork) {
+            priv->artwork = clutter_image_new();
+        }
+
+    priv->width = gdk_pixbuf_get_width (item->albumart);
+    priv->height = gdk_pixbuf_get_height (item->albumart);
+
+    if( priv->height > DEFAULT_IMG_SIZE) {
+        priv->width = priv->width * DEFAULT_IMG_SIZE / priv->height;
+        priv->height = DEFAULT_IMG_SIZE;
     }
 
     // Set cover artwork
-    gtk_clutter_texture_set_from_pixbuf (GTK_CLUTTER_TEXTURE(priv->texture), item->albumart, &error);
+    clutter_image_set_data( CLUTTER_IMAGE (priv->artwork),
+                                              gdk_pixbuf_get_pixels (item->albumart),
+                                              gdk_pixbuf_get_has_alpha (item->albumart)
+                                                      ? COGL_PIXEL_FORMAT_RGBA_8888
+                                                      : COGL_PIXEL_FORMAT_RGB_888,
+                                              priv->width,
+                                              priv->height,
+                                              gdk_pixbuf_get_rowstride (item->albumart),
+                                              &error);
     if (error) {
         g_warning("%s", error->message);
         g_error_free(error);
         return;
     }
 
-    // Add reflection
-    if (! priv->reflection) {
-        y_offset = clutter_actor_get_height (priv->texture) + V_PADDING;
+    // Set the width and height
+    clutter_actor_set_width(priv->texture, priv->width);
+    clutter_actor_set_height(priv->texture, priv->height);
+    clutter_actor_set_content (priv->texture, priv->artwork);
 
-        priv->reflection = clutter_clone_new (priv->texture);
-        clutter_actor_add_constraint (priv->reflection, clutter_bind_constraint_new (priv->texture, CLUTTER_BIND_X, 0.0));
-        clutter_actor_add_constraint (priv->reflection, clutter_bind_constraint_new (priv->texture, CLUTTER_BIND_Y, y_offset));
-        clutter_actor_add_constraint (priv->reflection, clutter_bind_constraint_new (priv->texture, CLUTTER_BIND_WIDTH, 0.0));
-        clutter_actor_add_constraint (priv->reflection, clutter_bind_constraint_new (priv->texture, CLUTTER_BIND_HEIGHT, 0.0));
-        g_signal_connect (priv->reflection,
-                       "paint",
-                       G_CALLBACK (_clone_paint_cb),
-                       NULL);
+    // Create the reflection
+    _create_reflection(self, item->albumart);
 
-        clutter_container_add_actor(CLUTTER_CONTAINER(self), priv->reflection);
-    }
-
-    ClutterActorBox box;
-    gfloat w, h;
-    clutter_actor_get_allocation_box (priv->texture, &box);
-    clutter_actor_box_get_size (&box, &w, &h);
-
-    if( h > DEFAULT_IMG_SIZE) {
-        gfloat temp = w * DEFAULT_IMG_SIZE / h;
-        clutter_actor_set_size(priv->texture, temp, DEFAULT_IMG_SIZE);
-    }
 
     // Add title / artist data
     if (priv->title)
@@ -285,44 +353,6 @@ void clarity_cover_set_album_item (ClarityCover *self, AlbumItem *item) {
     priv->artist = g_strdup(item->artist);
 }
 
-void clarity_cover_clear_rotation_behaviour(ClarityCover *self) {
-    g_return_if_fail(CLARITY_IS_COVER(self));
-
-    ClutterBehaviour *behaviour = self->priv->rotateBehaviour;
-
-    if (CLUTTER_IS_BEHAVIOUR(behaviour) && clutter_behaviour_is_applied(behaviour, CLUTTER_ACTOR(self))) {
-        clutter_behaviour_remove(behaviour, CLUTTER_ACTOR(self));
-        self->priv->rotateBehaviour = NULL;
-    }
-}
-
-void clarity_cover_set_rotation_behaviour(ClarityCover *self, ClutterAlpha *alpha, int final_angle, ClutterRotateDirection direction) {
-    g_return_if_fail(CLARITY_IS_COVER(self));
-
-    double current = clutter_actor_get_rotation(CLUTTER_ACTOR(self), CLUTTER_Y_AXIS, 0, 0, 0);
-
-    if(current < 0)
-        current += 360;
-    else if(current > 360)
-        current -= 360;
-
-    if(current != final_angle) {
-        self->priv->rotateBehaviour = clutter_behaviour_rotate_new (
-                alpha,
-                CLUTTER_Y_AXIS,
-                direction ,
-                current,
-                final_angle);
-
-        clutter_behaviour_rotate_set_center(
-                CLUTTER_BEHAVIOUR_ROTATE(self->priv->rotateBehaviour),
-                clutter_actor_get_width(CLUTTER_ACTOR(self)) / 2,
-                0, 0);
-
-        clutter_behaviour_apply (self->priv->rotateBehaviour, CLUTTER_ACTOR(self));
-    }
-}
-
 gchar *clarity_cover_get_title(ClarityCover *self) {
     ClarityCoverPrivate *priv = self->priv;
     return g_strdup(priv->title);
@@ -335,12 +365,12 @@ gchar *clarity_cover_get_artist(ClarityCover *self) {
 
 gfloat clarity_cover_get_artwork_height(ClarityCover *self) {
     ClarityCoverPrivate *priv = self->priv;
-    return clutter_actor_get_height(priv->texture);
+    return priv->height;
 }
 
 gfloat clarity_cover_get_artwork_width(ClarityCover *self) {
     ClarityCoverPrivate *priv = self->priv;
-    return clutter_actor_get_width(priv->texture);
+    return priv->width;
 }
 
 /**
