@@ -199,6 +199,9 @@ WebBrowser *init_web_browser(GtkWidget *parent) {
  * destroy the web browser and all associated items.
  */
 void destroy_cover_web() {
-    gtk_widget_destroy(browser->window);
+    if (GTK_IS_WIDGET(browser->window))
+        gtk_widget_destroy(browser->window);
+
+    g_free(browser);
     browser = NULL;
 }
