@@ -96,7 +96,7 @@ static gboolean pm_get_iter_for_playlist(Playlist *pl, GtkTreeIter *iter);
  *
  * ---------------------------------------------------------------- */
 static void pm_drag_begin(GtkWidget *widget, GdkDragContext *drag_context, gpointer user_data) {
-    /*     puts ("drag_begin"); */
+    /* nothing to do */
 }
 
 static void pm_drag_data_delete_remove_playlist(GtkTreeModel *tm, GtkTreePath *tp, GtkTreeIter *iter, gpointer data) {
@@ -113,8 +113,6 @@ static void pm_drag_data_delete(GtkWidget *widget, GdkDragContext *drag_context,
     g_return_if_fail (widget);
     g_return_if_fail (drag_context);
 
-    /*     printf ("drag_data_delete: %d\n", drag_context->action); */
-
     if (gdk_drag_context_get_selected_action(drag_context) == GDK_ACTION_MOVE) {
         GtkTreeSelection *ts = gtk_tree_view_get_selection(GTK_TREE_VIEW (widget));
         gtk_tree_selection_selected_foreach(ts, pm_drag_data_delete_remove_playlist, NULL);
@@ -124,8 +122,6 @@ static void pm_drag_data_delete(GtkWidget *widget, GdkDragContext *drag_context,
 static gboolean pm_drag_drop(GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, guint time, gpointer user_data) {
     GdkAtom target;
 
-    /*     puts ("drag_data_drop"); */
-
     gp_remove_autoscroll_row_timeout(widget);
 
     target = gtk_drag_dest_find_target(widget, drag_context, NULL);
@@ -138,13 +134,11 @@ static gboolean pm_drag_drop(GtkWidget *widget, GdkDragContext *drag_context, gi
 }
 
 static void pm_drag_end(GtkWidget *widget, GdkDragContext *drag_context, gpointer user_data) {
-    /*     puts ("drag_end"); */
     gp_remove_autoscroll_row_timeout(widget);
     gtkpod_tracks_statusbar_update();
 }
 
 static void pm_drag_leave(GtkWidget *widget, GdkDragContext *drag_context, guint time, gpointer user_data) {
-    /*     puts ("drag_leave"); */
     gp_remove_autoscroll_row_timeout(widget);
 }
 
@@ -164,7 +158,7 @@ static gboolean pm_drag_motion(GtkWidget *widget, GdkDragContext *dc, gint x, gi
     g_return_val_if_fail (widget, FALSE);
     g_return_val_if_fail (GTK_IS_TREE_VIEW (widget), FALSE);
 
-    gp_install_autoscroll_row_timeout(widget);
+    gp_install_autoscroll_row_timeout(widget, gdk_drag_context_get_device(dc));
 
     /* no drop possible if position is not valid */
     if (!gtk_tree_view_get_dest_row_at_pos(GTK_TREE_VIEW (widget), x, y, &path, &pos))
@@ -172,9 +166,6 @@ static gboolean pm_drag_motion(GtkWidget *widget, GdkDragContext *dc, gint x, gi
 
     g_return_val_if_fail (path, FALSE);
 
-    /*     printf ("pm_drag_motion (x/y/pos/s/a): %d %d %d %d %d\n", */
-    /* 	    x, y, pos, dc->suggested_action, dc->actions); */
-
     gtk_tree_view_set_drag_dest_row(GTK_TREE_VIEW (widget), path, pos);
 
     /* Get destination playlist in case it's needed */
@@ -351,10 +342,6 @@ static void pm_drag_data_get(GtkWidget *widget, GdkDragContext *dc, GtkSelection
     if (!data)
         return;
 
-    /*     puts ("data_get"); */
-
-    /* printf("sm drag get info: %d\n", info);*/
-
     ts = gtk_tree_view_get_selection(GTK_TREE_VIEW (widget));
     if (ts) {
         switch (info) {
@@ -387,7 +374,11 @@ static GdkDragAction pm_pm_get_action(Playlist *src, Playlist *dest, GtkWidget *
     g_return_val_if_fail (dc, 0);
 
     /* get modifier mask */
-    gdk_window_get_pointer(gtk_tree_view_get_bin_window(GTK_TREE_VIEW (widget)), NULL, NULL, &mask);
+    gdk_window_get_device_position(gtk_tree_view_get_bin_window(GTK_TREE_VIEW(widget)),
+                                                              gdk_drag_context_get_device(dc),
+                                                              NULL,
+                                                              NULL,
+                                                              &mask);
 
     /* don't allow copy/move before the MPL */
     if ((itdb_playlist_is_mpl(dest)) && (pos == GTK_TREE_VIEW_DROP_BEFORE))
@@ -471,7 +462,7 @@ static GdkDragAction pm_tm_get_action(Track *src, Playlist *dest, GtkTreeViewDro
 }
 
 /* Print a message about the number of tracks copied (the number of
- tracks moved is printed in tm_drag_data_delete() */
+ tracks moved is printed in tm_drag_data_delete()) */
 static void pm_tm_tracks_moved_or_copied(gchar *tracks, gboolean moved) {
     g_return_if_fail (tracks);
     if (!moved) {
@@ -489,6 +480,21 @@ static void pm_tm_tracks_moved_or_copied(gchar *tracks, gboolean moved) {
     }
 }
 
+static gint pm_adjust_for_drop_pos(gint position, GtkTreeViewDropPosition pos)
+{
+    switch (pos) {
+    case GTK_TREE_VIEW_DROP_BEFORE:
+    case GTK_TREE_VIEW_DROP_INTO_OR_BEFORE:
+        return position;
+    case GTK_TREE_VIEW_DROP_AFTER:
+    case GTK_TREE_VIEW_DROP_INTO_OR_AFTER:
+        return position + 1;
+    default:
+        g_warn_if_reached();
+        return position;
+    }
+}
+
 static void pm_drag_data_received(GtkWidget *widget, GdkDragContext *dc, gint x, gint y, GtkSelectionData *data, guint info, guint time, gpointer user_data) {
     GtkTreeIter iter_d, iter_s;
     GtkTreePath *path_d = NULL;
@@ -504,8 +510,6 @@ static void pm_drag_data_received(GtkWidget *widget, GdkDragContext *dc, gint x,
     Exporter *exporter = gtkpod_get_exporter();
     g_return_if_fail(exporter);
 
-    /* printf ("drag_data_received: x y a: %d %d %d\n", x, y, dc->suggested_action); */
-
     g_return_if_fail (widget);
     g_return_if_fail (dc);
     g_return_if_fail (data);
@@ -513,16 +517,15 @@ static void pm_drag_data_received(GtkWidget *widget, GdkDragContext *dc, gint x,
     g_return_if_fail (gtk_selection_data_get_data(data));
     g_return_if_fail (gtk_selection_data_get_format(data) == 8);
 
-    /* puts(gtk_tree_path_to_string (path)); */
-
     model = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
     g_return_if_fail (model);
 
     path_m = g_object_get_data(G_OBJECT (widget), "drag_data_by_motion_path");
 
-    if (path_m) { /* this callback was caused by pm_drag_motion -- we are
-     * supposed to call gdk_drag_status () */
-        /* puts ("...by motion"); */
+    if (path_m) {
+        /* this callback was caused by pm_drag_motion -- we are
+         * supposed to call gdk_drag_status () */
+
         pos = (GtkTreeViewDropPosition) g_object_get_data(G_OBJECT (widget), "drag_data_by_motion_pos");
         /* unset flag that */
         g_object_set_data(G_OBJECT (widget), "drag_data_by_motion_path", NULL);
@@ -543,9 +546,6 @@ static void pm_drag_data_received(GtkWidget *widget, GdkDragContext *dc, gint x,
                 g_return_if_reached ();
             }
             gdk_drag_status(dc, pm_tm_get_action(tr_s, pl, pos, dc), time);
-            /* 	    printf ("src: %p  dest: %p  sugg: %d a:%d\n", */
-            /* 		    pl_s->itdb, pl->itdb, dc->suggested_action, */
-            /* 		    pm_tm_get_action (tr_s, pl, pos, dc)); */
             return;
         case DND_GTKPOD_PLAYLISTLIST:
             /* get first playlist and check itdb */
@@ -555,18 +555,12 @@ static void pm_drag_data_received(GtkWidget *widget, GdkDragContext *dc, gint x,
                 g_return_if_reached ();
             }
             gdk_drag_status(dc, pm_pm_get_action(pl_s, pl, widget, pos, dc), time);
-            /* 	    printf ("src: %p  dest: %p  sugg: %d a:%d\n", */
-            /* 		    pl_s->itdb, pl->itdb, dc->suggested_action, */
-            /* 		    pm_pm_get_action (pl_s, pl, widget, pos, dc)); */
             return;
         }
         g_return_if_reached ();
         return;
     }
 
-    /*     printf ("treeview received drag data/length/format: %p/%d/%d\n", data, data?data->length:0, data?data->format:0); */
-    /*     printf ("treeview received drag context/actions/suggested action: %p/%d/%d\n", context, context?context->actions:0, context?context->suggested_action:0); */
-
     gp_remove_autoscroll_row_timeout(widget);
 
     path_ok = gtk_tree_view_get_dest_row_at_pos(GTK_TREE_VIEW(widget), x, y, &path_d, &pos);
@@ -590,7 +584,6 @@ static void pm_drag_data_received(GtkWidget *widget, GdkDragContext *dc, gint x,
 
     gchar *data_copy = g_strdup(gtk_selection_data_get_data(data));
 
-    /*  printf("position: %d\n", position); */
     switch (info) {
     case DND_GTKPOD_TRACKLIST:
         /* get first track */
@@ -626,10 +619,7 @@ static void pm_drag_data_received(GtkWidget *widget, GdkDragContext *dc, gint x,
         else { /* drop between playlists */
             Playlist *plitem;
             /* adjust position */
-            if (pos == GTK_TREE_VIEW_DROP_AFTER)
-                plitem = add_new_pl_user_name(pl->itdb, NULL, position + 1);
-            else
-                plitem = add_new_pl_user_name(pl->itdb, NULL, position);
+            plitem = add_new_pl_user_name(pl->itdb, NULL, pm_adjust_for_drop_pos(position, pos));
 
             if (plitem) {
                 /* copy files from iPod if necessary */
@@ -661,10 +651,8 @@ static void pm_drag_data_received(GtkWidget *widget, GdkDragContext *dc, gint x,
         }
         else { /* drop between playlists */
             Playlist *plitem;
-            if (pos == GTK_TREE_VIEW_DROP_AFTER)
-                plitem = add_text_plain_to_playlist(pl->itdb, NULL, data_copy, position + 1, NULL, NULL);
-            else
-                plitem = add_text_plain_to_playlist(pl->itdb, NULL, data_copy, position, NULL, NULL);
+            plitem = add_text_plain_to_playlist(pl->itdb, NULL, data_copy,
+                        pm_adjust_for_drop_pos(position, pos), NULL, NULL);
 
             if (plitem) {
                 gdk_drag_status(dc, GDK_ACTION_COPY, time);
@@ -700,12 +688,9 @@ static void pm_drag_data_received(GtkWidget *widget, GdkDragContext *dc, gint x,
                  * playlist */
                 switch (pos) {
                 case GTK_TREE_VIEW_DROP_BEFORE:
-                    pl_d = itdb_playlist_duplicate(pl_s);
-                    gp_playlist_add(pl->itdb, pl_d, position);
-                    break;
                 case GTK_TREE_VIEW_DROP_AFTER:
                     pl_d = itdb_playlist_duplicate(pl_s);
-                    gp_playlist_add(pl->itdb, pl_d, position + 1);
+                    gp_playlist_add(pl->itdb, pl_d, pm_adjust_for_drop_pos(position, pos));
                     break;
                 default:
                     pl_d = pl;
@@ -719,16 +704,6 @@ static void pm_drag_data_received(GtkWidget *widget, GdkDragContext *dc, gint x,
                 pm_get_iter_for_playlist(pl_s, &iter_s);
                 switch (pos) {
                 case GTK_TREE_VIEW_DROP_BEFORE:
-                    if (prefs_get_int("pm_sort") != SORT_NONE) {
-                        gtkpod_statusbar_message(_("Can't reorder sorted treeview."));
-                        gtk_drag_finish(dc, FALSE, FALSE, time);
-                        g_free(data_copy);
-                        return;
-                    }
-                    gtk_tree_store_move_before(GTK_TREE_STORE (model), &iter_s, &iter_d);
-                    pm_rows_reordered();
-                    gtk_drag_finish(dc, TRUE, FALSE, time);
-                    break;
                 case GTK_TREE_VIEW_DROP_AFTER:
                     if (prefs_get_int("pm_sort") != SORT_NONE) {
                         gtkpod_statusbar_message(_("Can't reorder sorted treeview."));
@@ -736,7 +711,11 @@ static void pm_drag_data_received(GtkWidget *widget, GdkDragContext *dc, gint x,
                         g_free(data_copy);
                         return;
                     }
-                    gtk_tree_store_move_after(GTK_TREE_STORE (model), &iter_s, &iter_d);
+                    if (pos == GTK_TREE_VIEW_DROP_BEFORE)
+                        gtk_tree_store_move_before(GTK_TREE_STORE (model), &iter_s, &iter_d);
+                    else
+                        gtk_tree_store_move_after(GTK_TREE_STORE (model), &iter_s, &iter_d);
+
                     pm_rows_reordered();
                     gtk_drag_finish(dc, TRUE, FALSE, time);
                     break;
@@ -755,16 +734,13 @@ static void pm_drag_data_received(GtkWidget *widget, GdkDragContext *dc, gint x,
             }
         }
         else { /*handle DND between two itdbs */
-            /* set destination pl */
             GList *trackglist = NULL;
-            pl_d = pl;
-            /* if drop is between two playlists, create new playlist */
-            /* FIXME: support copying of SPL data? */
-            if (pos == GTK_TREE_VIEW_DROP_BEFORE)
-                pl_d = gp_playlist_add_new(pl->itdb, pl_s->name, FALSE, position);
-            if (pos == GTK_TREE_VIEW_DROP_AFTER)
-                pl_d = gp_playlist_add_new(pl->itdb, pl_s->name, FALSE, position + 1);
+
+            /* create new playlist */
+            pl_d = gp_playlist_add_new(pl->itdb, pl_s->name, FALSE,
+                                       pm_adjust_for_drop_pos(position, pos));
             g_free(data_copy);
+            data_copy = NULL;
             g_return_if_fail (pl_d);
 
             /* copy files from iPod if necessary */
@@ -1524,10 +1500,22 @@ static void pm_cell_edited(GtkCellRendererText *renderer, const gchar *path_stri
     gtk_tree_model_get(model, &iter, PM_COLUMN_PLAYLIST, &playlist, -1);
     g_return_if_fail (playlist);
 
-    /*printf("pm_cell_edited: column: %d  track:%lx\n", PM_COLUMN_PLAYLIST, track);*/
-
     /* We only do something, if the name actually got changed */
     if (!playlist->name || g_utf8_collate(playlist->name, new_text) != 0) {
+
+        /*
+         * Check that the new name is not the same as that of another playlist
+         */
+        gchar *pl_name = g_strdup(new_text);
+        Playlist *pl = itdb_playlist_by_name(playlist->itdb, pl_name);
+        g_free(pl_name);
+
+        if (pl && pl != playlist) {
+            gtkpod_warning_simple(_("A playlist named '%s' already exists"), new_text);
+            // Stop the change as there is already a playlist with the same name
+            return;
+        }
+
         g_free(playlist->name);
         playlist->name = g_strdup(new_text);
         data_changed(playlist->itdb);
