@@ -86,9 +86,6 @@ static void set_default_preferences() {
         prefs_set_string("sm_sort_", NULL);
     }
 
-    if (!prefs_get_int_value("tm_autostore", NULL))
-        prefs_set_int("tm_autostore", TRUE);
-
     if (!prefs_get_int_value("tm_sortcol", NULL))
         prefs_set_int("tm_sortcol", TM_COLUMN_TITLE);
 
@@ -216,7 +213,7 @@ static void ipreferences_iface_init(IAnjutaPreferencesIface* iface) {
 
 static void track_command_iface_init(TrackCommandInterface *iface) {
     iface->id = "track_display_normalise_track_command";
-    iface->text = _("Normalise");
+    iface->text = _("Normalize");
     iface->execute = nm_tracks_list;
 }
 
