@@ -77,6 +77,9 @@ ENCODING=ISO-8859-15                      # encoding used by ipod
 # Corrections By Oliver Sherouse <oliver.sherouse AT gmail DOT com>
 # (replaced some arithmetic expressions, improved conversion, correctly
 # indicate "-d" in 'usage' instead of "-p")
+#
+# 2008/04/06 (Olivier Crete <tester at tester dot ca>):
+# Strip all tomboy tags with the power of sed
 
 # overwrite default settings with optional command line arguments
 while getopts i:d:e: option; do
@@ -110,7 +113,7 @@ if [ ! -d $NOTESPATH ]; then
 fi
 
 # check if iPod mountpoint exists
-if [ ! -d $IPOD_MOUNT/Notes ]; then
+if [ ! -d "$IPOD_MOUNT/Notes" ]; then
     echo "Error: Cannot find iPod at $IPOD_MOUNT"
     exit 1
 fi
@@ -120,7 +123,7 @@ fi
 # Seeing all notes should be under 4K, easier to delete and recopy rather than checking all files sizes
 
 echo -n "Syncing iPod ... [Notes] "
-rm -rf $IPOD_MOUNT/Notes/*
+rm -rf "$IPOD_MOUNT/Notes/*"
 rm -f $NOTESPATH/*~
 cd $NOTESPATH
 
@@ -138,19 +141,12 @@ ls *.note |
 	END=`echo $LIMITS | cut -d " " -f 2`
 
 	# Get the notes titles, we can use this as the filename
-	TITLE=`cat "$FILE" | cut -d "
-" -f $START | cut -d ">" -f 3`
-	
-	# Get the Lines with the useful data
-	DATA=`cat "$FILE" | cut -d "
-" -f $(( $START + 2 ))-$END`
+	TITLE=`cat "$FILE" | grep '<title>.*</title>' | sed -e 's:.*<title>\(.*\)</title>.*:\1:' | sed -e 's/<[^>]*>//g' -e 's/:/-/g'`
 
-	# Strip the tomboy stuff :(
-	for i in "<\/note-content>" "<\/text>" "<link:broken>" "<\/link:broken>" "<link:internal>" "<\/link:internal>" "<link:external>" "<\/link:external>" "<\/list>" "<list>" "<\/list-item>" "<list-item dir=\"ltr\">"; do
-		DATA=`printf "$DATA" | sed "s/$i//g"`
-	done
-
-	printf "$DATA" | $RECODE > "$IPOD_MOUNT/Notes/$TITLE"
+	cat "$FILE" | tail -n +2 | sed -e '/<\/note-content>/ Q' \
+		-e '/^[[:space:]]*<title>.*<\/title>[[:space:]]*$/ d' \
+		-e '/^[[:space:]]*<[^>]*>[[:space:]]*$/ d' \
+		-e 's/<[^>]*>//g'  | $RECODE > "$IPOD_MOUNT/Notes/$TITLE"
 
 	read FILE
   done
